/**
 * 
 */
package net.ildnpedia;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Prende in input un file delle guide di ildn e ne sostituisce i 
 * tags in modo da essere formattato per il wiki.
 * 
 * uso : 
 * 
 * java net.ildnpedia.Convert lista_di_files
 * 
 *  per ogni file nella lista viene creato il corrispondente .wiki
 * 
 * @author <a mailto="marco.ferretti@fermasoft.com">Marco Ferretti </a>
 * @version $Revision: 1.10 $
 * 
 */
public class Convert {
	/**
	 * Logger
	 */
	static Logger logger = null;
	/**
	 * Coda LIFO per i tags della guida ; serve per la gestione dei tags annidiati
	 */
	Vector divs = new Vector();
	String currentFile = null;
	static boolean containsErrors = false;
	
	public static void main(String[] args){
		System.out.println("Utility per la conversione delle guide ILDN. Versione 1.10");
		Convert converter = new Convert();
		File input = null;
		LogUtility.appendToExistingLog(false);
		LogUtility.useXMLFormatter(false);
		String[] filenames = converter.parseParams(args);
		logger = LogUtility.getFileLogger("logfile.log", Convert.class);
		logger.setLevel(Level.ALL);
		Logger resultLog = LogUtility.getFileLogger("results.log",Convert.class);
		resultLog.setLevel(Level.INFO);
		if ( filenames != null ) {
			for ( int i = 0; i < filenames.length ; ++i ){
				if ( (i%20) == 0  )
					System.gc(); // chiamo esplicitamente il garbage collector ogni 20 guide ...
				input = new File(filenames[i]);
				converter.currentFile = input.getAbsolutePath();
				converter.startLogging(input);
				try {
					converter.parseFile(input);
				resultLog.log(Level.INFO,"\nGuida : " + input.getName() + " : conversione avvenuta " + (containsErrors?"con":"senza") + " errori\n");
				} catch ( Exception e ) {
					logger.log(Level.SEVERE,e.getMessage());
					System.out.println("Errore di conversione per la guida : " + input.getName() );
					continue;
				}
			}
			System.exit(0);
		} else {
			System.err.println("Nessun file specificato !");
		}
	}
	

	/**
	 * Fa il parsing del file di input alla ricerca dei tags di formattazione delle guide.
	 *  
	 * @param input
	 */
	StringBuffer parseFile ( File input ) throws Exception {
		/*
		 * 
		 */
		StringBuffer work = null;
		try{
			work = read ( input );
		} catch ( Exception e ) {
			// non si può andare avanti ma il problema è già stato loggato
			throw ( e );
		}
		if ( hasDivTree(work)){
			logger.log(Level.WARNING,"La guida " + currentFile + " contiene divs annidiati !" );
		}
		work = parseDIV ( work );
		work = replaceClassicFormatting( work );
		work = replaceFontStyle( work );
		work = replaceURL( work );		
		work = parseCr ( work );
		work = handleBrInPre( work );
		write( work , input.getName() + ".wiki" );
		return work;
	}
	
	/**
	 * Legge il file di input 
	 * @param input
	 */
	public StringBuffer read( File input ) throws FileNotFoundException, IOException{
		logger.entering(currentFile,"read");
		StringBuffer buffer = new StringBuffer();
		String appoggio = null;
		try {
			FileReader reader = new FileReader(input);
			BufferedReader bufReader = new BufferedReader ( reader );
			while ( (appoggio = bufReader.readLine()) != null ){
				buffer.append(appoggio);
				buffer.append("\n");
			}
			return buffer;
		} catch (FileNotFoundException e) {
			logger.severe("File non trovato !");
			logger.log(Level.SEVERE, e.getMessage());
			containsErrors = true;
			throw (e);
		} catch (IOException e) {
			logger.severe("Errore io !");
			logger.log(Level.SEVERE, e.getMessage());
			containsErrors = true;
			throw ( e );
		} finally {
			logger.exiting(input.getAbsolutePath(),"read");
		}
	}
	/**
	 * Sostituisce i newline ( /n ) con &lt;br&gt;
	 * @param buffer
	 * @return
	 */
	StringBuffer parseCr ( StringBuffer buffer ) {
		logger.entering(currentFile,"parseCr");
		StringBuffer work = new StringBuffer ( buffer.toString() );
		int index = 0;
		
		while ( (index = work.indexOf("\n",index)) > -1 ) {
			work.replace(index,index + 1,"<br />");
		}
		logger.exiting(currentFile,"parseCr");
		return work;
	}
	
	/**
	 * Scrive il buffer
	 * @param out
	 * @param filename
	 */
	void write ( StringBuffer out , String filename )  {
		logger.entering(currentFile,"write");
		FileWriter writer;
		try {
			writer = new FileWriter(filename);
//			logger.log(Level.FINE,"About to write \n" + out.toString() );
			writer.write(out.toString());
			writer.write("\n");
			writer.flush();
			writer.close();
		} catch (IOException e) {
			logger.log(Level.SEVERE,e.getMessage());
			logger.exiting(currentFile,"write");
			containsErrors = true;
		}
		logger.exiting(currentFile,"write");
	}
	
	/**
	 * Sostituisce i tags div delle guide con i corrispondenti box di wiki
	 * @param buffer
	 * @return
	 */
	public StringBuffer parseDIV ( StringBuffer buffer ) {
		try {
			logger.entering(currentFile, "parseDiv");
			return new DIVWikifier(buffer, logger).evaluate();
		} catch ( RuntimeException e ) {
			// errore già loggato : non riesco a fare il parsing dei div
			// quindi ritorno la guida pari pari ...
			return buffer;
		} catch ( Exception e ) {
			// questi errori non sono stati loggati ...
			logger.log(Level.SEVERE, e.getMessage());
			containsErrors = true;
			return buffer;
		} finally {
			logger.exiting(currentFile, "parseDiv");
		}
	}
	
	/**
	 * Sostituisce i [b],[i] con gli apostrofi e i tags di chiusura delle
	 * guide che fanno riferimento ai font ( [/color],[/size],[/font] )
	 * @param buffer
	 * @return
	 */
	StringBuffer replaceClassicFormatting ( StringBuffer buffer ) {
		// le regex funziano ... è una questione di escape !
		try {
			logger.entering(currentFile,"replaceClassicFormatting");
			String temp = buffer.toString();
			temp = temp.replaceAll("\\[b\\]","'''");
			temp = temp.replaceAll("\\[/b\\]","'''");
			temp = temp.replaceAll("\\[i\\]","''");
			temp = temp.replaceAll("\\[/i\\]","''");
			temp = temp.replaceAll("\\[u\\]","<u>");
			temp = temp.replaceAll("\\[/u\\]","</u>");
			temp = temp.replaceAll("\\[code\\]","<pre>");
			temp = temp.replaceAll("\\[/code\\]","</pre>");
			temp = temp.replaceAll("\\[/color\\]","</font>");
			temp = temp.replaceAll("\\[/font\\]","</font>");		
			temp = temp.replaceAll("\\[/size\\]","</font>");
			buffer.delete ( 0, buffer.length() ) ;
			buffer.append(temp);
			logger.exiting(currentFile,"replaceClassicFormatting");
		} catch ( Exception e ) {
			logger.log(Level.SEVERE,e.getMessage());
			logger.exiting(currentFile,"replaceClassicFormatting");
			// è capitato qualcosa di strano ... ritorno quello che ho fatto finora
			containsErrors = true;
			return buffer;
		}
		return buffer;
	}
	
	/**
	 * Sostuisce i tags di apertura della guida che fanno riferimento 
	 * ai font ( [color],[size],[font] ) con il tag &lt;font&gt; del wiki
	 * @param buffer
	 * @return
	 */
	StringBuffer replaceFontStyle ( StringBuffer buffer ) {
		logger.entering(currentFile,"replaceFontStyle");
		int pos = -1;
		int prec = 0;
		StringBuffer work = new StringBuffer();
		StringBuffer temp = new StringBuffer();
		temp.append(buffer.substring(0,buffer.length()));
		try {
			while ( (pos = temp.indexOf("[color=",prec)) > -1 ) {
				work.append(buffer.substring(prec,pos));
				work.append("<font color=#");
				prec = pos + 7;
				work.append(temp.substring(prec, temp.indexOf("]",prec)));
				work.append(">");
				prec = temp.indexOf("]", prec ) +1 ;
			}
	
			if ( prec < temp.length() ) 
				work.append(temp.substring( prec ));
	
			prec = 0;
			temp.delete(0,work.length());
			temp.append(work.substring(0,work.length()));
			work.delete(0,work.length());
			while ( (pos = temp.indexOf("[size=",prec)) > -1 ) {
				work.append(temp.substring(prec,pos));
				work.append("<font size=");
				prec = pos + 6;
				work.append(temp.substring(prec, temp.indexOf("]",prec)));
				work.append(">");
				prec = temp.indexOf("]", prec ) +1 ;
			}
			if ( prec < temp.length() ) 
				work.append(temp.substring( prec ));
			
			prec = 0;
			temp.delete(0,work.length());
			temp.append(work.substring(0,work.length()));
			work.delete(0,work.length());
			while ( (pos = temp.indexOf("[font=",prec)) > -1 ) {
				work.append(temp.substring(prec,pos));
				work.append("<font face=");
				prec = pos + 6;
				work.append(temp.substring(prec, temp.indexOf("]",prec)));
				work.append(">");
				prec = temp.indexOf("]", prec ) +1 ;
			}
			
			if ( prec < temp.length() ) 
				work.append(temp.substring( prec ));
		} catch ( Exception e ){
			logger.log(Level.SEVERE, e.getMessage());
			logger.exiting(currentFile,"replaceFontStyle");
			containsErrors = true;
			return work;
		}
		logger.exiting(currentFile,"replaceFontStyle");
		return work;
	}
	
	/**
	 * Rimuove il tag [url] delle guide trasformandolo nel 
	 * relativo link per mediawiki
	 * @param buffer
	 * @return
	 */
	StringBuffer replaceURL ( StringBuffer buffer ) {
		// [url=http://a2.pluto.it/a2128.html#almlindex9361]link[/url]
		// diventa 
		// [http://a2.pluto.it/a2128.html#almlindex9361 link]
		String inizioURL = "[url=";
		String appoggio = null;
		int pos = 0;
		int last = pos;
		
		StringBuffer work = new StringBuffer();

		while ( ( pos = buffer.indexOf(inizioURL,last) ) > -1 ){
			work.append(buffer.substring(last,pos));
			work.append("[");
			last = buffer.indexOf("]",pos);
			work.append(buffer.substring(pos+inizioURL.length(),last)); // l'url
			work.append(" ");
			++last;
		}
		work.append ( buffer.substring(last) ) ; // tutto quello che rimane 
		// una volta fatta questa conversione non rimane che trasformare i [/url] in ]
		appoggio = work.toString();
		work = new StringBuffer( appoggio.replaceAll("\\[/url\\]","\\]") );
		return work;
	}
	
	/**
	 * Modifica tutti i &lt;br&bt; e i &lt;br /&gt; all'interno del tag &lt;pre&gt; 
	 * trasformandoli in /n
	 * @param buffer
	 * @return
	 */
	public StringBuffer handleBrInPre ( StringBuffer buffer ) {
		logger.entering(currentFile,"handleBrInPre");
		StringBuffer work = null;
		try {
			/*
			 * per lavorare un po' + velocemente faccio una magia ....
			 */
			String startMagic = "<abracadabra";
			String endMagic = "</abracadabra>";
			// trasformo tutti i <pre in qualcos'altro in modo da poter modificare tutto in una botta sola e 
			// poter maneggiare anche i tags annidiati
			String tmp = buffer.toString();
			tmp = tmp.replaceAll("<pre",startMagic);
			tmp = tmp.replaceAll("</pre>",endMagic);
//			StringBuffer work = new StringBuffer ( tmp );
			work = new StringBuffer ( tmp );
			StringBuffer appoggio = new StringBuffer (  );
	
			int da = 0;
			int a = 0;
			while ( work.indexOf(startMagic) > -1 ) { // more magic !
				da = work.lastIndexOf(startMagic);  // leggo l'inizio del tag
				a = work.indexOf(endMagic,da); // e la fine
				appoggio.append(work.substring(0,da)); // tutto quello che c'è prima del tag và eventualmente rielaborato
				tmp = work.substring(da,a+endMagic.length()); // metto tutto in una stringa in modo da poter utilizzare regex
				tmp = tmp.replaceAll("<br>","\n");// applico le regex
				tmp = tmp.replaceAll("<br />","\n");
				tmp = tmp.replaceAll(startMagic,"<pre");// e ritrasformo il tag in qualcosa che non riconsidero dopo
				tmp = tmp.replaceAll(endMagic,"</pre>");
				appoggio.append(tmp); // il risultato della trasformazione và nel buffer
				tmp = null; // paranoia setting
				appoggio.append(work.substring(a+endMagic.length())); // tutto quello che c'è dopo il tag và eventualmente rielaborato
				work.delete(0,work.length()); // sostituisco il buffer di lavoro
				work.append(appoggio.substring(0,appoggio.length()));
				appoggio.delete(0,appoggio.length()); // a cancello quello di appoggio
			}
		} catch ( Exception e ) {
			logger.log(Level.SEVERE,e.getMessage());
			containsErrors = true;
			logger.exiting(currentFile,"handleBrInPre");
			return work;
		}
		logger.exiting(currentFile,"handleBrInPre");
		return work;
	}
	
	/**
	 * Fà il parsing dei parametri : se sono files li ripassa pari pari ,
	 * se sono directory passa il contenuto delle directory
	 * @param args
	 * @return
	 */
	String[] parseParams ( String [] args ) {
		String[] ret = null;
		Vector appoggio = new Vector();
		File f = null;
		for ( int i = 0 ; i < args.length ; ++ i) {
			f = new File(args[i]);
			if ( f.exists() ) {
				// esiste . Se è un file non lo tocco, se è una directory entro 
				if ( f.isDirectory() ) {
					String[] temp = f.list();
					temp = expandDir(f.getAbsolutePath(),temp);
					for ( int j = 0 ; j < temp.length ; ++j ) 
						appoggio.addElement(temp[j]); // e metto in coda la lista dei files
				} else {
					appoggio.addElement(args[i]);
				}
			} else {
				// se è un percorso con una wildcard ( * ) provo a entrare 
				if ( args[i].trim().endsWith("*")){
					String[] tmp = { args[i].substring(0,args[i].lastIndexOf("*")) }; // creo un vettore con il percorso strippato
					tmp = expandDir(tmp[0], new String[]{""});
					for ( int j = 0 ; j < tmp.length ; ++j ) 
						appoggio.addElement(tmp[j]); // e metto in coda la lista dei files
				}
				
			}
		}
		if ( appoggio.size() > 0 ) {
			ret = new String[appoggio.size()];
			for ( int i = 0; i < appoggio.size() ; ++i ) 
				ret[i] = (String) appoggio.elementAt(i);
		}
		return ret;
	}
	
	/**
	 * Espande la directory corrente ( ricorsivamente ) e appende il basePath
	 * @param basePath
	 * @param list
	 * @return
	 */
	String[] expandDir ( String basePath , String[] list ) {
		String[] ret = null;
		File f = null;
		String fs = System.getProperty("file.separator");
		Vector appoggio = new Vector ( );
		if ( ! basePath.endsWith(fs) )
			basePath += fs;
		// scorro la lista dei files
		for ( int i = 0; i < list.length ; ++i ) {
			f = new File ( basePath + list[i]);
			if ( f.exists() ){ // se esiste
				if ( f.isDirectory() ) { // se è una directory
					String[] tmp = f.list(); // prendo il contenuto della directory
					tmp = expandDir(f.getAbsolutePath(),tmp); // e la espando
					if ( tmp != null )
						for ( int j = 0; j<tmp.length ; ++j )
							appoggio.addElement(tmp[j]);
				} else {
					appoggio.addElement(f.getAbsolutePath());
				}
			} 
		}
		
		if ( appoggio.size() > 0 ){
			ret = new String[appoggio.size()];
			for ( int i = 0; i < appoggio.size() ; ++i )
				ret[i] = (String) appoggio.elementAt(i);
		}
			
		return ret ;
	}
	
	void startLogging ( File input ) {
		containsErrors = false;
		logger.entering(input.getAbsolutePath(),"Inizio lavorazione");
	}
	public static void setError ( ) {
		containsErrors = true;
	}

	/**
	 * Ritorna true se nella guida sono presenti &lt;div&gt; annidiati
	 * @param buffer
	 * @return
	 */
	public boolean hasDivTree ( StringBuffer buffer ) {
		boolean ret = false;
		int inizio = 0;
		int fine = 0;
		int count = 0;
		int indice = 0;
		
		String inizioDiv = "<div";
		String fineDiv = "</div>";
//		String inizioCode = "[code]";
//		String fineCode = "[/code]";
		
		inizio = buffer.indexOf(inizioDiv);
		if ( inizio >= 0 ){ // la guida contiene div
			/*
			 * non cerco stringhe che iniziano per [code] perchè per quello che ne sò
			 * nelle guide non è possibile annidiare un <div> dentro un [code]... magari mi sbaglio
			 *  
			 */
			count = 1;
			fine = buffer.lastIndexOf(fineDiv);
			for ( indice = inizio + inizioDiv.length() ; indice < fine ; ++ indice ) { // il primo div l'ho già considerato ...
				// scorro la guida alla ricerca di inizioDiv,fineDiv e inizioCode e fineCode
				if ( count < 2 ){
					if ( buffer.substring(indice,indice + inizioDiv.length()).equalsIgnoreCase(inizioDiv) ) {
						++count;
						indice += inizioDiv.length();
					} else if ( buffer.substring(indice,indice + fineDiv.length()).equalsIgnoreCase(fineDiv) ){ 
							--count;
							indice += fineDiv.length();
//					} else if ( buffer.substring(indice,indice + inizioCode.length()).equalsIgnoreCase(inizioCode) ){ 
//						++count;
//						indice += inizioCode.length();
//					} else if ( buffer.substring(indice,indice + fineCode.length()).equalsIgnoreCase(fineCode) ){ 
//						--count;
//						indice += fineCode.length();
					}
					
				} else {
					// ci sono tags annidiati
					ret = true;
					break;
				}
			}
		}
		return ret ;
	}
	
//	boolean hasDivTree2 ( StringBuffer guida ) {
//		boolean ret = false;
//		
//		
//		String inizioDiv = "<div";
//		String fineDiv = "</div>";
//		String inizioCode = "[code]";
//		
//		int inizio = guida.indexOf(inizioDiv);
//		int next = -1;
//		int inizioC = -1;
//		int fine = -1;
//		
//		while ( inizio > -1 ) { // devo controllare a partire da questo <div 
//			next = guida.indexOf(inizioDiv,inizio + inizioDiv.length());
//			if ( next > -1 ) { // ho + di un <div nella guida
//				fine = guida.substring(inizio,next).indexOf(fineDiv);
//				if ( fine < 0 || fine > next) { // sono annidiati ?
//					ret = true;
//					break;	// esco
//				}
//			}
//			// potrei avere un [code] all'interno di un <div ...
//			inizioC =  guida.indexOf(inizioCode,inizio + inizioDiv.length());
//			if ( inizioC > -1 ){
//				fine = guida.substring(inizio,inizioC).indexOf(fineDiv);
//				if ( fine < 0 ) { // sono annidiati ?
//					ret = true;
//					break;	// esco
//				}
//			}
//			// se sono quì o non ho ancora trovato un div annidiato 
//			// o non ci sono altri div ... ma questo lo controllo dopo :
//			inizio = next;
//		}
//		return ret;
//	}
//	
//	
}
