/**
 * 
 */
package net.ildnpedia;

import java.io.IOException;
import java.util.logging.FileHandler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.logging.XMLFormatter;

/**
 * Questa classe implementa un logger
 * 
 * 
 * @author <a mailto="marco.ferretti@fermasoft.com">Marco Ferretti </a>
 * @version $Revision: 1.3 $
 * 
 */
public class LogUtility {

	static boolean appendToExistingLog = true;

	static boolean useXMLFormatter = false;

	/**
	 * Ritorna un FileHandler per la classe . Il file di log ha lo 
	 * stesso nome della classe con l'estensione .log
	 * @param aClass
	 * @return
	 */
	public static FileHandler getLogFileHandler(Class aClass) {
		String className = aClass.getName();
		int lastIndex = className.lastIndexOf(".");
		if (lastIndex >= 0)
			className = className.substring(lastIndex + 1);
		return getLogFileHandler(className + ".log");
	}

	/**
	 * Ritorna un FileHandler che punta al file con il nome specificato come parametro 
	 * @param filename
	 * @return
	 */
	public static FileHandler getLogFileHandler(String filename) {
		FileHandler handler = null;
		try {
			handler = new FileHandler(filename, isAppendingToExistingLog());
			if (isUsingXMLFormatter())
				handler.setFormatter(new XMLFormatter());
			else
				handler.setFormatter(new SimpleFormatter());
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return handler;
	}

	/**
	 * Attiva/disattiva l'uso di xmlFormatter per i log files che sono 
	 * instanziati a partire dalla chiamata a questo metodo
	 * @param useIt
	 */
	public static void useXMLFormatter(boolean useIt) {
		useXMLFormatter = useIt;
	}

	/**
	 * Dice se il formatter corrente è XMLFormatter
	 * @return
	 */
	public static boolean isUsingXMLFormatter() {
		return useXMLFormatter;
	}

	/**
	 * Setta il flag di append/overwrite per il file di log
	 * @param append
	 */
	public static void appendToExistingLog(boolean append) {
		appendToExistingLog = append;
	}

	/**
	 * Dice se il log file viene sovrascritto o appeso
	 * @return
	 */
	public static boolean isAppendingToExistingLog() {
		return appendToExistingLog;
	}

	/**
	 * Ritorna un logger per la classe passata come parametro 
	 * @param aClass
	 * @return
	 */
	public static Logger getFileLogger(Class aClass) {
		Logger logger = Logger.getLogger(aClass.getName());
		FileHandler logHandler = LogUtility.getLogFileHandler(aClass);
		if (logHandler != null) {
			logger.addHandler(logHandler);
		} else {
			logger.info("cannot redirect output to logfile");
		}

		return logger;
	}
	/**
	 * Ritorna un logger per la classe passata come parametro che punta al file passato come parametro 
	 * @param aClass
	 * @param filename
	 * @return
	 */
	public static Logger getFileLogger(String filename, Class aClass) {
		Logger logger = Logger.getLogger(aClass.getName());
		FileHandler logHandler = LogUtility.getLogFileHandler(filename);
		if (logHandler != null) {
			logger.addHandler(logHandler);
		} else {
			logger.info("cannot redirect output to logfile");
		}

		return logger;
	}

}
